#----------------------------------------------------------------------------------#
#
# FILE:			__init__.py
# AUTHOR:		Joe Sapp
# DATE:			22.05.2006
# VERSION:		1.00
#
# DESCRIPTION:	RSS News grabber
#
# INTERFACE:	browser			RW	Set the browser to be used by the launcher.
#				read			W	Reads RSS feed from provider.
#				launch			R	Launch URL within a browser.
#				status			R	Status of the reader. (0 = Idle, 1 = Reading)
#				items			R	Get the list of news items.
#
#----------------------------------------------------------------------------------#

from libdesklets.controls import Control, Interface, Permission

import re, commands, os
import rssparser

from NewsItem import NewsItem

#----------------------------------------------------------------------------------#
#
# INewsGrab Interface
#
#
class IRSSPicker(Interface):

	status		= Permission.READ
	items		= Permission.READ
	type		= Permission.READ
	launch		= Permission.WRITE
	read		= Permission.WRITE
	browser		= Permission.READWRITE


#----------------------------------------------------------------------------------#
#
# RSSPicker Class
#
#
class RSSPicker(Control, IRSSPicker):

	#
	# Object initialization
	#
	def __init__(self):
		
		self._browser	= "/usr/bin/firefox"
		self._url		= ""
		self._items		= []
		self._status	= 0
		self.__type		= 0
		
		Control.__init__(self)


	#
	# Some getter & setter methods
	#
	def __set_browser(self, value):	self._browser = value
	def __get_browser(self):		return self._browser
	def __get_items(self):			return self._items
	def __get_status(self):			return self._status


	#
	# Force a read from the RSS provider
	#
	def __read_feed(self, url):

		self._url = url
		self._status = 1
		self._update("status")

		feed = rssparser.parse(url)

		del self._items[0:len(self._items)]
		self._items = [NewsItem() for i in range(len(feed['items'])+1)]

		# Get channel information
		self._items[0].link  = feed['channel'].get('link', url)
		self._items[0].title = feed['channel'].get('title', "")
		self._items[0].description = feed['channel'].get('description', "")

		for f in range(len(feed['items'])):
			# Each item is a dictionary mapping properties to values
			self._items[f+1].link  = feed['items'][f].get('link', "about:none")
			self._items[f+1].title = feed['items'][f].get('title', "")
			self._items[f+1].description = feed['items'][f].get('description', "")

		self._status = 0
		self._update("status")


	#
	# Gets the type of XML feed
	#
	def __get_type(self):

		return self.__type
   
		
	#
	# Launch the URL within the browser specified by set_browser()
	#
	def __launch(self, value):
		
                url=value
                while(re.search("&amp;amp;", url)):
                  url = url.replace("&amp;amp;", '&')
                while(re.search("&amp;", url)):
                  url = url.replace("&amp;", '&')
                os.system("nice -n 0 %s '%s' &" % (self._browser, url))		
	
	#
	# Interface
	#
	type			= property(	fget = __get_type,
								doc = "Set the type of XML feed (0 = RSS, 1 = ATOM")
	browser			= property(	fset = __set_browser, fget = __get_browser,
								doc = "Set the default browser")
	launch			= property(	fset = __launch,
								doc = "Launch the URL within a browser")
	items			= property(	fget = __get_items,
								doc = "Get the news items array")
	read			= property(	fset = __read_feed,
								doc = "Read RSS feed from provider")
	status			= property(	fget = __get_status,
								doc = "Thread Status (0 = Idle, 1 = Reading RSS)")


#----------------------------------------------------------------------------------#

def get_class(): return RSSPicker
